// GPIOSample.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Windev.h"

#include "gpio.h"

#define GPIO_86		86	//pin no. 7 in P16 (SB-x270)
#define GPIO_107	107	//pin no. 45 in p16 (SB-x270)
#define GPIO_104	104 //pin no. 6 in p16 (SB-x270)
/************************************************************************************
This short program demonstrates how to use Compulab's GPIO Driver.
************************************************************************************/


int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	int iRet = TRUE;
	HANDLE hEvent; 
	GPIO_VALUE gpioVal;
	
	//Opens a handle to the gpio driver
	HANDLE hGpio = GPIOOpen();
	if (hGpio == NULL)
		return FALSE;

	//Check if the gpio we are about to use are available for us
	if(!GPIOIsAvailable(hGpio, GPIO_86) || !GPIOIsAvailable(hGpio,GPIO_104) || !GPIOIsAvailable(hGpio,GPIO_107))
	{
		printf("The gpios selected are not available\n");
		return 0;
	}
		
	//sets gpio 107 as Input
	GPIOSetAsInput(hGpio, GPIO_107);

	//sets gpio 86 as Output with a logical value of 1
	GPIOSetAsOutput(hGpio, GPIO_86, GPIO_HIGH);
	


	//Sets up an interrupt event for GPIO 107 to react on falling and rising edge
		
	hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	printf("Event hnd = 0x%x.\n",hEvent);

	//Initialize the GPIO_INTERRUPT struct
	GPIO_INTERRUPT intr;
	intr.dwGpioNum = GPIO_107;			
	intr.Edge = GPIO_REACT_ON_BOTH;		//Decides if we react on falling edge, rising edge or both.
	intr.Event = hEvent;				//The event the will be signaled when the edge will happen.
	iRet = GPIORegisterInterrupt(hGpio, intr);
	
	GPIO_VALUE gpioVal86 = GPIO_HIGH;
	//This loop will continue until you trigger the edge event	
	while(TRUE)
	{
		//waits for the edge event we just initialized. 
		iRet = WaitForSingleObject(hEvent, 3000);

		if (iRet == WAIT_TIMEOUT)
		{
			//no edge was detected	
			printf("GPIO: Interrupt timeout. Trigger an edge for gpio 107 to continue\n");
			//Invert the value of GPIO 86
			GPIOSetValue(hGpio, GPIO_86, gpioVal86);
			gpioVal86 = (gpioVal86 == GPIO_HIGH) ? GPIO_LOW : GPIO_HIGH;
		}
		else
		{
			//edge was detected. we can proceed 
			printf("GPIO: Interrupt event was signaled.\n");
			//This function will signal the kernel that interrupt processing has been completed.
			GPIOInterruptDone(hGpio, GPIO_107);
			break;
		}
	}

	//Unregisters the interrupt on gpio 107
	GPIOUnRegisterInterrupt(hGpio, GPIO_107);

	//This section demonstrates polling.
	//the loop will run until the logical value of gpio 104 is 0
	gpioVal = GPIO_HIGH;
	GPIOSetAsInput(hGpio, GPIO_104);
	printf("Polling GPIO 104, Assert a logical value 1 to continue.\n");
	while(gpioVal != GPIO_LOW)
	{
		if(GPIOGetValue(hGpio, GPIO_104, &gpioVal) == FALSE)
		{
			printf("Error: Unable to read gpio 104 value\n");
			break;
		}
		Sleep(100);		
	}
	printf("That concludes the GPIO sample.\n");
	//Close the handle to the gpio driver
	GPIOClose(hGpio);

	return 0;
}

